/*
 * Copyright 2012 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1
import Friends 0.1

Item {
    id: tile
    height: status_update_content.height + detailsWrapper.height
    width: parent.width
    property real detailsOpacity : 0
    property string service: column_0
    property string accountId: column_1
    property string messageId: column_2
    property string sender: column_4
    property string timeString: friendsUtils.createTimeString(column_8)
    property string message: column_9
    property string avatar: column_10
    property string url: column_11
    property int likes: column_12 > 0 ? column_12 : liked ? 1 : 0
    property bool liked: column_13
    property string link_picture: column_14
    property string link_name: column_15
    property string link_url: column_16
    property string link_desc: column_17
    property string link_caption: column_18
    property string link_icon: column_19
    property string location: column_20
    property string latitude: column_21
    property string longitude: column_22
    property string likesString: likes > 1 ? i18n.tr("people liked this") : i18n.tr("person liked this")

    anchors {
        topMargin: units.gu(1)
        leftMargin: units.gu(1)
        bottomMargin: units.gu(1)
        rightMargin: units.gu(1)
    }

    FriendsUtils {
        id: friendsUtils
    }

    FriendsDispatcher {
        id: friends
        onLikeComplete: {
             if (success) {
                 liked = true;
                 favoriteIcon.opacity = 1.0;
             } else {
                 console.log ("Like failed: " + errorMessage);
             }
         }
        onUnlikeComplete: {
             if (success) {
                 liked = false;
                 favoriteIcon.opacity = 0.3;
             } else {
                 console.log ("UnLike failed: " + errorMessage);
             }
         }
    }

    UbuntuShape {
        id: avatarImage
        height: units.dp(48)
        width: units.dp(48)
        anchors {
            left: parent.left
            top: parent.top
            leftMargin: units.gu(1)
            topMargin: units.gu(1)
        }
        image: Image {
            source: Qt.resolvedUrl(avatar)
            asynchronous: true
            fillMode: Image.PreserveAspectFit
        }

        MouseArea {
            anchors.fill: avatarImage
            onClicked: {
                Qt.openUrlExternally(url);
            }
        }
    }

    Item {
        id: status_update_content
        anchors {
            left: avatarImage.right
            right: parent.right
            top: parent.top
            topMargin: units.gu(1)
            leftMargin: units.gu(1)
            bottomMargin: units.gu(1)
            rightMargin: units.gu(1)
        }
        height: childrenRect.height

        MouseArea {
            anchors.fill: status_update_content
            onClicked: {
                if (tile.state != 'Details') {
                    tile.state = 'Details';
                } else {
                    tile.state = 'List';
                }
            }
        }

        Column {
            spacing: units.gu(1)
            anchors {
                left: parent.left;
                right: parent.right
                bottomMargin: units.gu(2)
            }
            Label {
                text: sender
                fontSize: "medium"
                font.bold: true
                width: parent.width
            }
            Label {
                id: messageText
                text: message
                visible: message.length > 0
                wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                width: parent.width - units.gu(3)
                onLinkActivated: Qt.openUrlExternally(link)
            }
            Label {
                id: locationName
                text: location
                visible: location.length > 0
                wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                width: parent.width - units.gu(3)
                MouseArea {
                    anchors.fill: locationName
                    onClicked: {
                        if (url.length > 0)
                        {
                            Qt.openUrlExternally(url);
                        }
                    }
                }
            }
            Label {
                id: linkName
                text: link_name
                visible: link_name.length > 0
                wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                width: parent.width - units.gu(3)
                MouseArea {
                    anchors.fill: linkName
                    onClicked: {
                        if (link_url.length > 0)
                            Qt.openUrlExternally(link_url);

                    }
                }
            }
            Label {
                id: linkDesc
                text: link_desc
                visible: link_desc.length > 0
                wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                width: parent.width - units.gu(3)
                onLinkActivated: Qt.openUrlExternally(link)
            }

            ListItem.Empty {
                id: favorite
                width: parent.width - avatarImage.width
                height: favoriteIcon.height + units.gu(1)
                anchors {
                    left: parent.left
                    right: parent.right
                    bottomMargin: units.gu(1)
                }
                showDivider: false
                Image {
                    id: favoriteIcon
                    source: "images/favorite.png"
                    opacity: liked ? 1.0 : 0.3
                    width: 16
                    anchors {
                        left: parent.left
                        right: likesLabel.left
                    }
                }
                MouseArea {
                    anchors.fill: favoriteIcon
                    onClicked: {
                        if (liked) {
                            friends.unlikeAsync(accountId, messageId);
                        }
                        else {
                            friends.likeAsync(accountId, messageId);
                        }
                    }
                }
                Label {
                    id: likesLabel
                    text: (likes > 0) ? likes + " " + likesString : ""
                    width: parent.width - favoriteIcon.width
                    anchors {
                        left: favoriteIcon.right
                        right: serviceIcon.right
                    }
                    fontSize: "small"
                    color: "gray"
                }

                Image {
                    id: serviceIcon
                    anchors {
                        right: parent.right
                    }
                    source: "images/"+service+".png"
                    width: 22
                    height: 22
                    asynchronous: true
                }
            }
        }

        Row {
            anchors.right: parent.right
            anchors.rightMargin: units.gu(1)
            anchors.top: parent.top
            spacing: units.gu(1)

            Label {
                id: time
                text: timeString
                fontSize: "small"
                color: "gray"
            }
        }
    }

    states: [State {
                name: "Details"
                PropertyChanges { target: tile; detailsOpacity: 1; x: 0 }
                /* Looks like we don't need to resize to grow, but we do for reducing
                PropertyChanges { target: content; height: status_update_content.height + detailsWrapper.height}
                */
                StateChangeScript {
                    script: {
                        listView.positionViewAtIndex(listView.indexAt(parent.x, parent.y) - 1, ListView.Contain);
                    }
                }

            },
            State {
                name: "List"
                PropertyChanges { target: content; height: status_update_content.height }
                PropertyChanges { target: tile; detailsOpacity: 0; x: 0 }
            }]


    transitions: Transition {
        NumberAnimation {
            duration: 300; properties: "detailsOpacity,x,height,width"
        }
    }


    Rectangle {
        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
            bottom: parent.bottom
        }
        color: "black"
        opacity: 0.1
        visible: detailsOpacity > 0 ? true : false
    }

    Item {
        id: detailsWrapper
        height: detailsLoader.height
        width: parent.width
        opacity: detailsOpacity
        visible:  detailsOpacity > 0 ? true : false

        anchors {
            top: status_update_content.bottom
            left: parent.left
            right: parent.right
        }

        Loader {
            id: detailsLoader
            width: parent.width
        }

        onVisibleChanged: {
            if (visible && detailsLoader.state != Loader.Ready) {
                detailsLoader.source = "StatusUpdateTileDetails.qml";
            }
        }
    }
}
